<?php
namespace Codexonics\PrimeMoverFramework\utilities;

/*
 * This file is part of the Codexonics.PrimeMoverFramework package.
 *
 * (c) Codexonics Ltd
 *
 * This package is Open Source Software. For the full copyright and license
 * information, please view the LICENSE file which was distributed with this
 * source code.
 */

use Codexonics\PrimeMoverFramework\classes\PrimeMoverImporter;

if (! defined('ABSPATH')) {
    exit;
}
/**
 *
/**
 * Extended Duplicator Search Replace Class
 * Standalone and removed reporting, logging methods.
 *
 * @package PrimeMoverFramework\utilities
 * @link https://github.com/lifeinthegrid/duplicator Duplicator GitHub Project
 * @link http://www.lifeinthegrid.com/duplicator/
 * @link http://www.snapcreek.com/duplicator/
 * @author Snap Creek
 * @author Codexonics
 * @copyright 2011-2017  SnapCreek LLC
 * @license GPLv2 or later

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 * SOURCE CONTRIBUTORS:
 * David Coveney of Interconnect IT Ltd
 * https://github.com/interconnectit/Search-Replace-DB/
 */
final class PrimeMoverSearchReplace extends DupxUpdateEngine
{
    /**
     * 
     * @param $dbh	
     * @param array $list
     * @param array $tables
     * @param boolean $fullsearch
     * @param PrimeMoverImporter $importer
     * @param array $excluded_columns
     * @compatible 5.6
     */
    public static function load($dbh, $list = array(), $tables = array(), $fullsearch = false, PrimeMoverImporter $importer = null, $excluded_columns = [])
    {
        if (! $importer->getSystemAuthorization()->isUserAuthorized()) {
            return;
        }
        
        try {
            $walk_function = function () use (&$str) {
                $str = "`$str`";
            };
        } catch (\Exception $exc) {
        }

        $table_with_excluded_column = '';
        $excluded_column = '';
        if (is_array($excluded_columns) && ! empty($excluded_columns) ) {
            $table_with_excluded_column = key($excluded_columns);
            $excluded_column = reset($excluded_columns);
        }
        
        if (is_array($tables) && !empty($tables)) {
            foreach ($tables as $table) {
                $columns = array();

                // Get a list of columns in this table
                $fields	 = mysqli_query($dbh, 'DESCRIBE '.$table);
                while ($column	 = mysqli_fetch_array($fields)) {
                    $columns[$column['Field']] = $column['Key'] == 'PRI' ? true : false;
                }
                
                // Excluded column
                if ($table_with_excluded_column === $table && array_key_exists($excluded_column, $columns)) {
                    unset($columns[$excluded_column]);
                }
                
                // Count the number of rows we have in the table if large we'll split into blocks
                $row_count	 = mysqli_query($dbh, "SELECT COUNT(*) FROM `{$table}`");
                $rows_result = mysqli_fetch_array($row_count);
                @mysqli_free_result($row_count);
                $row_count	 = $rows_result[0];
                if ($row_count == 0) {
                    continue;
                }

                $page_size	 = 25000;
                $offset		 = ($page_size + 1);
                $pages		 = ceil($row_count / $page_size);

                // Grab the columns of the table.  Only grab text based columns because
                // they are the only data types that should allow any type of search/replace logic
                $colList = '*';
                $colMsg	 = '*';
                if (!$fullsearch) {
                    $colList = self::getTextColumns($dbh, $table);
                    if ($colList != null && is_array($colList)) {
                        array_walk($colList, $walk_function);
                        $colList = implode(',', $colList);
                    }
                    $colMsg = (empty($colList)) ? '*' : '~';
                }

                if (empty($colList)) {
                    continue;
                } else {
                }

                //Paged Records
                for ($page = 0; $page < $pages; $page++) {
                    $current_row = 0;
                    $start		 = $page * $page_size;
                    $end		 = $start + $page_size;
                    $sql		 = sprintf("SELECT {$colList} FROM `%s` LIMIT %d, %d", $table, $start, $offset);
                    $data		 = mysqli_query($dbh, $sql);

                    $scan_count = ($row_count < $end) ? $row_count : $end;
                    

                    //Loops every row
                    while ($row = mysqli_fetch_array($data)) {
                        $current_row++;
                        $upd_col	 = array();
                        $upd_sql	 = array();
                        $where_sql	 = array();
                        $upd		 = false;
                        $serial_err	 = 0;
                        $is_unkeyed = !in_array(true, $columns);

                        //Loops every cell
                        foreach ($columns as $column => $primary_key) {
                            $edited_data		= $data_to_fix = $row[$column];
                            $base64converted	= false;
                            $txt_found			= false;

                            //Unkeyed table code
                            //Added this here to add all columns to $where_sql
                            //The if statement with $txt_found would skip additional columns
                            if ($is_unkeyed && ! empty($data_to_fix)) {
                                $where_sql[] = $column.' = "'.mysqli_real_escape_string($dbh, $data_to_fix).'"';
                            }

                            //Only replacing string values
                            if (!empty($row[$column]) && !is_numeric($row[$column]) && $primary_key != 1) {
                                //Base 64 detection
                                if (base64_decode($row[$column], true)) {
                                    $decoded = base64_decode($row[$column], true);
                                    if (self::isSerialized($decoded)) {
                                        $edited_data	 = $decoded;
                                        $base64converted	 = true;
                                    }
                                }

                                //Skip table cell if match not found
                                foreach ($list as $item) {
                                    if (strpos($edited_data, $item['search']) !== false) {
                                        $txt_found = true;
                                        break;
                                    }
                                }
                                if (!$txt_found) {
                                    continue;
                                }

                                //Replace logic - level 1: simple check on any string or serialized strings
                                foreach ($list as $item) {
                                    $edited_data = self::recursiveUnserializeReplace($item['search'], $item['replace'], $edited_data);
                                }

                                //Replace logic - level 2: repair serialized strings that have become broken
                                $serial_check = self::fixSerialString($edited_data);
                                if ($serial_check['fixed']) {
                                    $edited_data = $serial_check['data'];
                                } elseif ($serial_check['tried'] && !$serial_check['fixed']) {
                                    $serial_err++;
                                }
                            }

                            //Change was made
                            if ($edited_data != $data_to_fix || $serial_err > 0) {
                                //Base 64 encode
                                if ($base64converted) {
                                    $edited_data = base64_encode($edited_data);
                                }
                                $upd_col[]	 = $column;
                                $upd_sql[]	 = $column.' = "'.mysqli_real_escape_string($dbh, $edited_data).'"';
                                $upd		 = true;
                            }

                            if ($primary_key) {
                                $where_sql[] = $column.' = "'.mysqli_real_escape_string($dbh, $data_to_fix).'"';
                            }
                        }
                        if ($upd && !empty($where_sql)) {
                            $sql	= "UPDATE `{$table}` SET ".implode(', ', $upd_sql).' WHERE '.implode(' AND ', array_filter($where_sql));
                            $result	= mysqli_query($dbh, $sql);
                        }
                    }
                    @mysqli_free_result($data);
                }
            }
        }
    }
}
